﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Tools.Ribbon;
using Microsoft.Office.Interop.Excel;
using Excel = Microsoft.Office.Interop.Excel;
using System.Net;
using System.IO;
using System.Windows.Forms;
using System.Drawing;
using System.Globalization;

using MimoX.excepcao;
using MimoX.util;
using MimoX.visao;
using MimoX.servico;
using MimoX.Properties;

namespace MimoX
{

    public partial class Ribbon
    {
        private JsonUtil jsonUtil;
        private RequisicaoHttpUtil requisicaoHttpUtil;
        private MensagemService mensagemService;
        private MensagemForm mensagemForm;
        private CreditoForm creditoForm;
        private MimoIdForm mimoIdForm;
        private Acerca acercaForm;
        private Settings settings;
        private NumberFormatInfo formatador = new CultureInfo("en-US", false).NumberFormat;
        
        public static string TOKEN { get; set; }

        private void Ribbon1_Load(object sender, RibbonUIEventArgs e)
        {
            inicializarAtributosObrigatorios();
        }       

        private void button1_Click(object sender, RibbonControlEventArgs e)
        {
            AutenticacaoForm autenticacaoForm = new AutenticacaoForm();
            autenticacaoForm.setRibbon(this);

            autenticacaoForm.mostrarJanela();
        }

        private void btnEnviarSms_Click(object sender, RibbonControlEventArgs e)
        {
            mensagemForm.mostrarJanelaParaEnviarSms();
          
        }

        private void btnRequisitar_Click(object sender, RibbonControlEventArgs e)
        {
            mimoIdForm.mostrarJanela();
            
        }

        private void btnCarregarSms_Click(object sender, RibbonControlEventArgs e)
        {
            creditoForm.mostrarJanela();
        }
        
        public void verificarSeExisteTokenGuardadoSenaoBloquear()
        {
            try
            {
                string token = FicheiroUtil.lerConteudoDoFicheiro();
                Boolean temToken = token != null;
                if (temToken)
                {
                    string resultado = requisicaoHttpUtil.executarRequisicaoGet(@"http://52.30.114.86:8080/mimosms/v1/user/exist?token=" + token);
                    temToken = resultado.Contains("true");

                    tornarPaineisVisiveisExceptoLogin(temToken);
                    if (temToken)
                    {
                        TOKEN = token;
                        Properties.Settings.Default.token = token;
                        Properties.Settings.Default.Save();

                        buscarDadosDoUtilizador();
                    }
                }
                else
                {
                    tornarPaineisVisiveisExceptoLogin(false);
                }

            }
            catch (Exception)
            {
                tornarPaineisVisiveisExceptoLogin(false);
            }
        }

        private void tornarPaineisVisiveisExceptoLogin(Boolean invisivel)
        {
            GrupologIn.Visible = !invisivel;
            GrupoEnviarSMS.Visible = invisivel;
            GrupoCreditos.Visible = invisivel;
            GrupoSenderID.Visible = invisivel;
            GrupoTerminarSessao.Visible = invisivel;
        }

        private void inicializarAtributosObrigatorios()
        {
            requisicaoHttpUtil = new RequisicaoHttpUtil();
            mensagemForm = new MensagemForm();
            mensagemForm.setRibbon(this);

            creditoForm = new CreditoForm();
            creditoForm.setRibbon(this);

            mimoIdForm = new MimoIdForm();

            acercaForm = new Acerca();

            jsonUtil = new JsonUtil();
            settings = new Settings();
            
            mensagemService = new MensagemService();
            formatador.NumberDecimalDigits = 0;

            verificarSeExisteTokenGuardadoSenaoBloquear();
        }

        private void buscarDadosDoUtilizador()
        {
            string resultado = requisicaoHttpUtil.executarRequisicaoGet(@"http://52.30.114.86:8080/mimosms/v1/user/list-one?token=" + TOKEN);

            if (requisicaoHttpUtil.sucesso)
            {
                Utilizador utilizador = new Utilizador();
                utilizador = (Utilizador)jsonUtil.toObject(resultado, utilizador.GetType());
                Properties.Settings.Default.nomeDoUtilizador = utilizador.name;
                Properties.Settings.Default.Save();
                Properties.Settings.Default.Save();
                salvarCreditosNoFicheiroDePropriedades(utilizador.credits.Replace("Unlimited", "Pós-pago"));
            }
            else
            {
                MensagemUtil.mostrarMensagemDeErro(ExcepcaoHandler.tratarMensagemDeErroDeLogin(resultado));
            }
        }

        public void salvarCreditosNoFicheiroDePropriedades(string creditos)
        {
            if (creditos.Contains("Pós-pago"))
            {
                GrupoCreditos.Visible = false;
            }
            else
            {
                Int64 creditosAux = Int64.Parse(creditos);
                creditos = creditosAux.ToString("N", formatador);
            }
            Properties.Settings.Default.meusCreditos = creditos + " SMS";
            Properties.Settings.Default.Save();
            this.groupAcerca.RibbonUI.Invalidate();
           
        }

        public void consultarCreditos()
        {
            string resultado = requisicaoHttpUtil.executarRequisicaoGet(@"http://52.30.114.86:8080/mimosms/v1/credit?token=" + TOKEN);

            if (requisicaoHttpUtil.sucesso)
            {
                Credito credito = new Credito();
                credito = (Credito)jsonUtil.toObject(resultado, credito.GetType());

                salvarCreditosNoFicheiroDePropriedades(credito.balance.Replace("Unlimited", "Pós-pago"));
            }
            else
            {
                MensagemUtil.mostrarMensagemDeErro(resultado);
            }
        }

        private void button1_Click_1(object sender, RibbonControlEventArgs e)
        {
            acercaForm.ShowDialog();
        }

        private void button2_Click(object sender, RibbonControlEventArgs e)
        {
            string resultado = requisicaoHttpUtil.executarRequisicaoGet(@"http://52.30.114.86:8080/mimosms/v1/user/logout?token=" + TOKEN);

            if (requisicaoHttpUtil.sucesso)
            {
                Properties.Settings.Default.nomeDoUtilizador = "Nome do utilizador";
                Properties.Settings.Default.meusCreditos = "0 SMS";
                Properties.Settings.Default.Save();
                FicheiroUtil.guardarConteudoNoFicheiro("");
                tornarPaineisVisiveisExceptoLogin(false);
            }
            else
            {
                MensagemUtil.mostrarMensagemDeErro(resultado);
            }
        }
    }
}
