﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Net.NetworkInformation;
using Microsoft.Win32;

using MimoX.visao;
using MimoX.util;
using MimoX.constante;
using MimoX.excepcao;

namespace MimoX
{
    public partial class AutenticacaoForm : Form
    {
        private RequisicaoHttpUtil requisicaoHttpUtil;
        private JsonUtil jsonUtil;
        private Ribbon ribbon;

        public AutenticacaoForm()
        {
            InitializeComponent();
            requisicaoHttpUtil = new RequisicaoHttpUtil();
            jsonUtil = new JsonUtil();

            botaoEntrar.BackColor = cores.AZUL1;
            linkLabelRegistar.LinkColor = cores.AZUL2;
            linkLabelRecuperarPassword.LinkColor = cores.AZUL2;
            
        }

        public void setRibbon(Ribbon ribbon)
        {
            this.ribbon = ribbon;
        }
        
        private void button1_Click(object sender, EventArgs e)

        {   
            entrar();

        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            RequisicaoHttpUtil.abrirLinkNoNavegador("http://www.mimo.it.ao/pt/");
           
        }

        private void linkLabelRecuperarPassword_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            
            RequisicaoHttpUtil.abrirLinkNoNavegador("http://www.mimo.it.ao/pt/programadores");
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            base.OnFormClosing(e);

            if (e.CloseReason == CloseReason.WindowsShutDown) return;

            e.Cancel = true;
            this.Hide();
        }


        private void entrar()
        {
            if (!requisicaoHttpUtil.temInternet())
            {
                MensagemUtil.mostrarMensagemDeErro("Para utilizar este serviço precisa estar ligado à internet");
                return;
            }
            //if (!requisicaoHttpUtil.fazerPingNoServidorDeSMS())
            //{
            //    MensagemUtil.mostrarMensagemDeErro("Noão foi possível estabelecer ligação com o servidor");
            //    return;
            //}
            string utilizador = txtUtilizador.Text.Trim();
            string password = txtPassword.Text.Trim();

            if (utilizador.Length == 0)
            {
                MensagemUtil.mostrarMensagemDeErro("Informe o utilizador");
                return;
            }

            Regex regex = new Regex(@"^9[0-9]{8}$");
            if (!regex.Match(utilizador).Success)
            {
                MensagemUtil.mostrarMensagemDeErro("Utilizador inválido");
                return;
            }

            if (password.Length == 0)
            {
                MensagemUtil.mostrarMensagemDeErro("Informe a palavra-passe");
                return;
            }

            if (password.Length < 6)
            {
                MensagemUtil.mostrarMensagemDeErro("A palavra-passe deve ter pelo menos 6 caracteres");
                return;
            }
            Autenticacao autenticacao = new Autenticacao(utilizador, password);
            string json = jsonUtil.toJson(autenticacao);

            StringBuilder url = new StringBuilder()
                .Append("http://52.30.114.86:8080/mimosms/v1/user/login")
                .Append("?reference=").Append(GetMacAddress())
                .Append("&name=Excel&version=").Append(getExcelVersion())
                .Append("&type=").Append("Microsoft Office")
                .Append("&ip=")
                .Append(getLocalIPAddress())
                .Append("&operatingSystem=").Append(Environment.OSVersion);
            
            string retorno = requisicaoHttpUtil.executarRequisicaoPost(@url.ToString(), json);

            if (requisicaoHttpUtil.sucesso)
            {
                AccessToken accessToken = new AccessToken();
                accessToken = (AccessToken)jsonUtil.toObject(retorno, accessToken.GetType());
                Ribbon.TOKEN = accessToken.token;
                FicheiroUtil.guardarConteudoNoFicheiro(accessToken.token);
                ribbon.verificarSeExisteTokenGuardadoSenaoBloquear();
                this.Dispose(true);
                limparTela();
            }
            else
            {
                MensagemUtil.mostrarMensagemDeErro(ExcepcaoHandler.tratarMensagemDeErroDeLogin(retorno));
                txtPassword.Text = string.Empty;
            }
        }

        private void limparTela()
        {
            txtUtilizador.Text = string.Empty;
            txtPassword.Text = string.Empty;
        }
        
        private void txtPassword_KeyPress(object sender, KeyPressEventArgs e)
        {
            if(e.KeyChar == (char)Keys.Enter)
            {
                entrar();
            }
        }

        private void txtUtilizador_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                entrar();
            }
        }

        public void mostrarJanela()
        {
            txtUtilizador.Focus();
            ShowDialog();
            txtUtilizador.Focus();
        }

        private IPAddress getLocalIPAddress()
        {
            IPHostEntry host = Dns.GetHostEntry(Dns.GetHostName());

            return host
                .AddressList
                .FirstOrDefault(ip => ip.AddressFamily == AddressFamily.InterNetwork);
        }

        public string GetMacAddress()
        {
            NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();
            String sMacAddress = string.Empty;
            foreach (NetworkInterface adapter in nics)
            {
                if (sMacAddress == String.Empty)
                {
                    IPInterfaceProperties properties = adapter.GetIPProperties();
                    sMacAddress = adapter.GetPhysicalAddress().ToString();
                }
            }
            return sMacAddress;
        }

        private string getExcelVersion()
        {
            string version = Globals.ThisAddIn.GetApplication().Version.ToString();
            switch (version)
            {
                case "7.0":
                    return "95";
                case "8.0":
                    return "97";
                case "9.0":
                    return "2000";
                case "10.0":
                    return "2002";
                case "11.0":
                    return "2003";
                case "12.0":
                    return "2007";
                case "14.0":
                    return "2010";
                case "15.0":
                    return "2013";
                case "16.0":
                    return "2016";
                case "19.0":
                    return "2019";
                default:
                    return version;
            }
        }
        
    }
}
