﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using MimoX.util;
using MimoX.visao;
using MimoX.constante;
using MimoX.excepcao;


namespace MimoX
{
    public partial class CreditoForm : Form
    {
        private JsonUtil jsonUtil;
        private RequisicaoHttpUtil requisicaoHttpUtil;
        private Ribbon ribbon;

        public CreditoForm()
        {
            InitializeComponent();

            inicializarAtributosObrigatorios();

            botaoCarregarConta.BackColor = cores.AZUL1;
            infoRecarga.LinkColor = cores.AZUL2;

            txtVoucher.PromptChar = ' ';
        }
        
        private void button1_Click(object sender, EventArgs e)
        {
            carregarConta();
        }
        private void inicializarAtributosObrigatorios()
        {

            requisicaoHttpUtil = new RequisicaoHttpUtil();
            jsonUtil = new JsonUtil();

        }
        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            base.OnFormClosing(e);

            if (e.CloseReason == CloseReason.WindowsShutDown) return;

            e.Cancel = true;

            limparTela();
            this.Hide();
        }

        public void setRibbon(Ribbon ribbon)
        {
            this.ribbon = ribbon;
        }

        public void carregarConta()
        {

            string voucher = txtVoucher.Text.Trim();
            Regex reg = new Regex(@"[0-9]{14}");
            if (voucher.Length != 14 || !reg.Match(voucher).Success)
            {
                MensagemUtil.mostrarMensagemDeErro("Voucher inválido");
               
                return;
            }

            string resultado = requisicaoHttpUtil.executarRequisicaoGet(@"http://52.30.114.86:8080/mimosms/v1/credit/recharge?token=" + Ribbon.TOKEN + "&voucher=" + voucher);

            if (requisicaoHttpUtil.sucesso)
            {
                Carregamento credito = new Carregamento();
                credito = (Carregamento)jsonUtil.toObject(resultado, credito.GetType());

                ribbon.salvarCreditosNoFicheiroDePropriedades("" + credito.currentCredits);

                MensagemUtil.mostrarMensagemDeSucesso("Carregamento efectuado com sucesso.\nVocê tem " + credito.currentCredits + " SMSs");
                limparTela();
                this.Close();
            }
            else
            {
                MensagemUtil.mostrarMensagemDeErro(ExcepcaoHandler.tratarMensagemDeErroDeCredito(resultado));
            }
        }

        
        public void consultarCreditos()
        {
            string resultado = requisicaoHttpUtil.executarRequisicaoGet(@"http://52.30.114.86:8080/mimosms/v1/credit?token=" + Ribbon.TOKEN);

            if (requisicaoHttpUtil.sucesso)
            {
                Credito credito = new Credito();
                credito = (Credito)jsonUtil.toObject(resultado, credito.GetType());

                ribbon.salvarCreditosNoFicheiroDePropriedades(credito.balance.Replace("Unlimited", "Pós-pago"));
                //MessageBox.Show(credito.balance.Contains("limited") ? "Você tem SMS Ilimitado" : "Você tem " + credito.balance + " SMSs");
            }
            else
            {
                MessageBox.Show(resultado);
            }
        }

        private void limparTela()
        {
            txtVoucher.Text = string.Empty;
        }
        
        public void mostrarJanela()
        {
            ShowDialog();
            txtVoucher.Focus();
        }

        private void txtVoucher_KeyPress(object sender, KeyPressEventArgs e)
        {
            if(e.KeyChar == (char)Keys.Enter)
            {
                carregarConta();
            }
        }
    }
}
