﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using MimoX.util;
using MimoX.visao;
using MimoX.constante;
using MimoX.servico;

namespace MimoX
{
    public partial class MensagemForm : Form
    {
        private JsonUtil jsonUtil;
        private RequisicaoHttpUtil requisicaoHttpUtil;
        private MensagemService mensagemService;
        private int colunaSelecionadaParaOsDestinatarios;
        private RequisicaoHttpUtil enviarMensagem = new RequisicaoHttpUtil();
        private Ribbon ribbon;


        public MensagemForm()
        {
            InitializeComponent();
            txtMensagem.MaxLength = MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_SEM_CARACTERE_ESPECIAL;

            requisicaoHttpUtil = new RequisicaoHttpUtil();
            jsonUtil = new JsonUtil();

            mensagemService = new MensagemService();
            mensagemService.mensagemForm = this;
            botaoEnviar.BackColor = cores.AZUL1;
            lblCreditos.BackColor = cores.AZUL1;

            buscarMimoIDs();
            
        }

        public void setRibbon(Ribbon ribbon)
        {
            this.ribbon = ribbon;
        }

        public Ribbon getRibbon()
        {
            return ribbon;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            EnviarMensagem();
            barraDeProgresso.Visible = false;
        }
        
        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            base.OnFormClosing(e);

            if (e.CloseReason == CloseReason.WindowsShutDown) return;

            e.Cancel = true;
            this.Hide();
            
        }

        private void checkBoxAgendamento_CheckedChanged(object sender, EventArgs e)
        {
            txtAgendamento.Visible = checkBoxAgendamento.Checked;
            txtAgendamento.MaxDate = DateTime.Now.AddDays(6);
            txtAgendamento.MinDate = DateTime.Now;
            txtAgendamento.Value = DateTime.Now;
        }

        private void EnviarMensagem()
        {
            barraDeProgresso.Visible = true;
            string sender = comboMimoID.Text.Trim();
            string mensagem = txtMensagem.Text.Trim();
            if (sender.Length == 0)
            {
                MensagemUtil.mostrarMensagemDeErro("Selecione o Sender ID");
                return;
            }

            if (mensagem.Length == 0)
            {
                MensagemUtil.mostrarMensagemDeErro("Informe o conteúdo da mensagem");
                return;
            }

            if (colunaSelecionadaParaOsDestinatarios <= 0)
            {
                MensagemUtil.mostrarMensagemDeErro("Selecione a coluna dos destinatários do ficheiro");
                return;
            }

            if (checkBoxAgendamento.Checked)
            {
                TimeSpan timeSpan = txtAgendamento.Value.Subtract(DateTime.Now);
                if (timeSpan.TotalMinutes < 0)
                {
                    MensagemUtil.mostrarMensagemDeErro("A data e hora do agendamento deve ser superior a data e hora actual (" + DateTime.Now.ToString("yyyy-MM-dd HH:mm") + ")");
                    return;
                }
            }

            mensagemService.enviarMensagem(comboMimoID.Text, colunaSelecionadaParaOsDestinatarios, mensagem, checkBoxtTags.Checked, txtAgendamento.Value, checkBoxAgendamento.Checked);
            
        }

        public void limparTela()
        {
            barraDeProgresso.Value = 0;
            setTexto(string.Empty);
            checkBoxtTags.Checked = false;
            labelAlertaTags.Visible = false;
            txtAgendamento.MaxDate = DateTime.Now.AddDays(6);
            txtAgendamento.MinDate = DateTime.Now;
            txtAgendamento.Value = DateTime.Now;
            buscarMimoIDs();
            adicionarNomesDasColunasNasComboBoxesDestinatariosETags();
        }

        public void setTexto(string texto)
        {
            txtMensagem.Text = texto;
        }

        public void buscarMimoIDs()
        {
            try
            {
                Pagina<MimoId> pagina = new Pagina<MimoId>();
                string json = requisicaoHttpUtil.executarRequisicaoGet("http://52.30.114.86:8080/mimosms/v1/sender-id/list-all?token=" + Ribbon.TOKEN + "&paginate=false");
                pagina = (Pagina<MimoId>)jsonUtil.toObject(json, pagina.GetType());
                comboMimoID.Items.Clear();
                MimoId pattern = null;
                foreach (MimoId mimoId in pagina.content)
                {
                    comboMimoID.Items.Add(mimoId.sender);
                    if(mimoId.pattern)
                    {
                        pattern = mimoId;
                    }
                }

                if (comboMimoID.Items.Count == 0)
                {
                    comboMimoID.Items.Add("MIMO");
                    comboMimoID.SelectedIndex = 0;
                } else
                {
                    comboMimoID.SelectedItem = pattern.sender;
                }

            }
            catch (System.Exception)
            {
                comboMimoID.Items.Clear();
                comboMimoID.Items.Add("MIMO");
                comboMimoID.SelectedIndex = 0;
            }
            
        }

        public void mostrarJanelaParaEnviarSms()
        {
            limparTela();
            comboDestinatarios.Focus();
            ShowDialog();
        }

        private void adicionarNomesDasColunasNasComboBoxesDestinatariosETags()
        {
            comboDestinatarios.SelectedIndex = -1;
           // comboTags.SelectedIndex = -1;
            comboDestinatarios.Items.Clear();
            //comboTags.Items.Clear();
            foreach (string coluna in mensagemService.carregarNomesDasColunas())
            {
                comboDestinatarios.Items.Add(coluna);
            }
        }

        private void comboDestinatarios_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            colunaSelecionadaParaOsDestinatarios = comboDestinatarios.SelectedIndex + 1;
        }

        public void actualizarValorDaProgressBar(int valor)
        {
            barraDeProgresso.Value = valor;
            barraDeProgresso.CreateGraphics().DrawString(valor + "%",
                new Font("Arial", (float)8.25, FontStyle.Regular),
                Brushes.Black,
                new PointF(barraDeProgresso.Width / 2 - 10, barraDeProgresso.Height / 2 - 7));
        }

        private void txtMensagem_TextChanged(object sender, EventArgs e)
        {
            string texto = txtMensagem.Text;

            int totalCaracteres = MensagemValidator.isBinary(texto) ? MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_COM_CARACTERE_ESPECIAL - texto.Length : MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_SEM_CARACTERE_ESPECIAL - texto.Length;

            if (!atingiuTamanhoMaximo(texto.Trim()))
            {
                actulaizarContadorDeMensagem(null, labelTotalDeMensagens, texto, totalCaracteres);
            }
            else
            {

            }
        }

        private Boolean atingiuTamanhoMaximo(String texto)
        {
            int totalCaracteres = MensagemValidator.isBinary(texto) ? MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_COM_CARACTERE_ESPECIAL - texto.Length : MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_SEM_CARACTERE_ESPECIAL - texto.Length;

            return totalCaracteres < 0;
        }

        private void actulaizarContadorDeMensagem(Label labelEnviarSmsCaracteresRestantes, Label labelEnviarSmsParte, String texto, int totalCaracteres)
        {
            totalCaracteres = MensagemValidator.isBinary(texto) ? MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_COM_CARACTERE_ESPECIAL - texto.Length : MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_SEM_CARACTERE_ESPECIAL - texto.Length;

            //labelEnviarSmsCaracteresRestantes.setText = texto.isEmpty() ? "" + MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_SEM_CARACTERE_ESPECIAL : totalCaracteres.ToString());

            labelTotalDeMensagens.Text = texto.Length == 0 ? MensagemValidator.TAMANHO_DA_MENSAGEM_VAZIA + "/10 SMS" : getTotalDeMensagem(texto) + "/10 SMS";
        }

        private int getTotalDeMensagem(String mensagem)
        {
            String texto = mensagem;
            if (MensagemValidator.isBinary(texto))
            {
                txtMensagem.MaxLength = MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_COM_CARACTERE_ESPECIAL;
                return texto.Length <= 70 ? 1 : (texto.Length / 67) + ((texto.Length % 67 == 0) ? 0 : 1);
            }
            else
            {
                txtMensagem.MaxLength = MensagemValidator.TAMANHO_MAXIMO_DA_MENSAGEM_SEM_CARACTERE_ESPECIAL;
                return texto.Length <= 160 ? 1 : (texto.Length / 153) + ((texto.Length % 153 == 0) ? 0 : 1);
            }
        }

        public class MensagemValidator
        {
            private static String ALFA_NUMERICO = "[a-zA-Z0-9 àÉéèùìòÇÖÑÄÜàäèéñöÉüÅ]+";
            private static String SINAIS_DE_PONTUACAO = "[;:!,]";
            private static String SINAIS_ESPECIAIS = "[Æ¤€¡Øæø¿ßΦΠΣΩΨΓΘΔ]";
            private static String SIMBOLOS = "[-_<>=%#\"'~@£§$&/]";
            private static String SIMBOLOS_ER = "[\\n\\?\\.\\*\\+\\\\(\\)\\[\\]\\{\\}\\^\\|]+";
            private static String CARACTERES_UNICODE = "[" + ALFA_NUMERICO + SINAIS_DE_PONTUACAO + SIMBOLOS + SIMBOLOS_ER + SINAIS_ESPECIAIS + "]+";
            private static Regex regex = new Regex(CARACTERES_UNICODE);
            public static int TAMANHO_MAXIMO_DA_MENSAGEM_COM_CARACTERE_ESPECIAL = 670;
            public static int TAMANHO_MAXIMO_DA_MENSAGEM_SEM_CARACTERE_ESPECIAL = 1530;
            public static int TAMANHO_DA_MENSAGEM_VAZIA = 0;
            public static String CREDITOS = " SMS";
            public static String SUFIXO_TOTAL_DE_MENSAGENS = " /10 SMS";
            public static String SUFIXO_CARACTERES_RESTANTES = " Caracteres restantes";

            public static Boolean isBinary(String mensagem)
            {
                return !regex.Match(mensagem).Success;
            }

        }
        
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            labelAlertaTags.Visible = checkBoxtTags.Checked;
        }   
        
    }
}
