﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Office.Interop.Excel;
using System.Text.RegularExpressions;
using System.Web;
using MimoX.visao;
using MimoX.util;
using MimoX.excepcao;


namespace MimoX.servico
{
    class MensagemService
    {
        private RequisicaoHttpUtil requisicaoHttpUtil;
        private JsonUtil jsonUtil;
        private StringBuilder stringBuilder;
        private List<FormacaoMap> formacoesMap;
        Regex regexTelemovel = new Regex(@"(\+2449|002449|2449|9){1}[0-9]{8}");

        public MensagemForm mensagemForm { get; set; }
        private Int64 contadorDeEnvioDeMensagemPersonalizado = 0;
        private Int64 contadorDeEnvioDeMensagemPersonalizadoParaDestinatariosIncorretos = 0;

        public MensagemService()
        {
            requisicaoHttpUtil = new RequisicaoHttpUtil();
            jsonUtil = new JsonUtil();
        }

        public void enviarMensagem(string mimoId, int colunaSelecionadaParaOsDestinatarios, string texto, Boolean textoEstaFormatado, DateTime agendamento, Boolean estaAgendada)
        {
            stringBuilder = new StringBuilder();
            Worksheet worksheet = Globals.ThisAddIn.GetActiveWorksheet();

            Range range = worksheet.UsedRange;

            carregarDadosDoExcel(range, colunaSelecionadaParaOsDestinatarios, mimoId, texto, textoEstaFormatado, agendamento, estaAgendada);

        }

        private void carregarDadosDoExcel(Range range, int colunaSelecionadaParaOsDestinatarios, string mimoId, string texto, Boolean textoEstaFormatado, DateTime dateTime, Boolean estaAgendada)
        {
            int totalDelinhas = range.Rows.Count;
            int totalDeColunas = range.Columns.Count;
            int i = 1;

            Regex regex = new Regex(@"##\w*##");

            if (textoEstaFormatado && regex.Match(texto).Success)
            {
                mensagemForm.actualizarValorDaProgressBar(0);
                MatchCollection matches = regex.Matches(texto);

                List<string> formatacoes = new List<string>();

                foreach (Match match in matches)
                {
                    formatacoes.Add(match.Value);
                }

                while (i <= totalDelinhas)
                {
                    if (range.Item[i, colunaSelecionadaParaOsDestinatarios] != null)
                    {
                        Range item = range.Item[i, colunaSelecionadaParaOsDestinatarios];
                        string telemovel = item.Text;
                        if (regexTelemovel.Match(telemovel).Success)
                        {
                            StringBuilder textoFormatado = new StringBuilder()
                                .Append(texto);
                            foreach (string formatacao in formatacoes)
                            {
                                foreach (FormacaoMap map in formacoesMap)
                                {
                                    if (map.formato.Equals(formatacao))
                                    {
                                        textoFormatado.Replace(formatacao, range.Item[i, map.indice].Text);
                                        break;
                                    }
                                }
                            }

                            enviarMensagem(mimoId, telemovel, textoFormatado.ToString(), estaAgendada, dateTime);
                            contadorDeEnvioDeMensagemPersonalizado++;
                        }
                        else
                        {
                            contadorDeEnvioDeMensagemPersonalizadoParaDestinatariosIncorretos++;
                            item.Font.Color = System.Drawing.ColorTranslator.ToOle(System.Drawing.Color.Red);
                        }
                    }
                    mensagemForm.actualizarValorDaProgressBar(getPercentegem(i, totalDelinhas));
                    i++;
                }
                if (contadorDeEnvioDeMensagemPersonalizado > 0 && contadorDeEnvioDeMensagemPersonalizadoParaDestinatariosIncorretos > 0)
                {
                    MensagemUtil.mostrarMensagemDeSucesso("Mensagem enviadas para " + contadorDeEnvioDeMensagemPersonalizado + " destinatários");
                    mensagemForm.limparTela();
                }
                else if (contadorDeEnvioDeMensagemPersonalizado > 0)
                {
                    mensagemForm.limparTela();
                    MensagemUtil.mostrarMensagemDeSucesso("Mensagem enviada com sucesso");
                } else if(contadorDeEnvioDeMensagemPersonalizadoParaDestinatariosIncorretos > 0)
                {
                    MensagemUtil.mostrarMensagemDeErro("Mensagem não enviada", "Os números dos destinatários estão inválidos");
                }
                else 
                {
                    MensagemUtil.mostrarMensagemDeErro("Mensagem não enviada", "Não foi possível enviar as mensagens");
                }
                contadorDeEnvioDeMensagemPersonalizado = 0;
                contadorDeEnvioDeMensagemPersonalizadoParaDestinatariosIncorretos = 0;
            }
            else
            {
                List<string> destinatarios = new List<string>();
                while (i <= totalDelinhas)
                {
                    if (range.Item[i, colunaSelecionadaParaOsDestinatarios] != null)
                    {
                        Range item = range.Item[i, colunaSelecionadaParaOsDestinatarios];
                        string telemovel = item.Text;
                        if (regexTelemovel.Match(telemovel).Success)
                        {
                            stringBuilder.Append(telemovel);
                            stringBuilder.Append(",");
                        }
                        else
                        {
                            item.Font.Color = System.Drawing.ColorTranslator.ToOle(System.Drawing.Color.Red);
                        }
                    }
                    mensagemForm.actualizarValorDaProgressBar(getPercentegem(i, totalDelinhas));
                    i++;
                }
                stringBuilder.Remove(stringBuilder.Length - 1, 1);
                try
                {
                    MensagemPost mensagemPost = new MensagemPost(mimoId, stringBuilder.ToString(), HttpUtility.UrlEncode(texto, System.Text.Encoding.UTF8));
                    mensagemPost.groups = "";
                    mensagemPost.schedule = estaAgendada ? dateTime.ToString("yyyyMMddHHmm") : "";
                    string json = jsonUtil.toJson(mensagemPost);

                    string retorno = requisicaoHttpUtil.executarRequisicaoPost(@"http://52.30.114.86:8080/mimosms/v1/message/send?token=" + Ribbon.TOKEN, json);

                    if (requisicaoHttpUtil.sucesso)
                    {
                        mensagemForm.limparTela();
                        MensagemUtil.mostrarMensagemDeSucesso("Mensagem enviada com sucesso!!!");
                    }
                    else
                    {
                        MensagemUtil.mostrarMensagemDeErro("Mensagem não enviada", ExcepcaoHandler.tratarMensagemDeErroDeSms(retorno));
                    }
                }
                catch (System.Net.WebException ex)
                {
                    MensagemUtil.mostrarMensagemDeErro("Mensagem não enviada", ex.Message);
                }
                mensagemForm.actualizarValorDaProgressBar(0);
                mensagemForm.getRibbon().consultarCreditos();

            }

        }

        private void enviarMensagem(String mimoId, string telemovel, string texto, Boolean estaAgendada, DateTime dateTime)
        {
            try
            {
                StringBuilder url = new StringBuilder(@"http://52.30.114.86:8080/mimosms/v1/message/send?token=")
                .Append(Ribbon.TOKEN)
                .Append("&sender=")
                .Append(mimoId)
                .Append("&recipients=")
                .Append(telemovel)
                .Append("&text=")
                .Append(HttpUtility.UrlEncode(texto, System.Text.Encoding.UTF8))
                .Append(estaAgendada ? "&schedule=" + dateTime.ToString("yyyyMMddHHmm") : "");

                string retorno = requisicaoHttpUtil.executarRequisicaoGet(url.ToString());

                if (requisicaoHttpUtil.sucesso)
                {
                    //MensagemUtil.mostrarMensagemDeSucesso("Mensagem enviada com sucesso!!!");
                }
                else
                {
                    //MensagemUtil.mostrarMensagemDeErro("Mensagem não enviada", retorno);
                }
            }
            catch (Exception)
            {

            }

        }

        public List<string> carregarNomesDasColunas()
        {
            formacoesMap = new List<FormacaoMap>();
            Worksheet worksheet = Globals.ThisAddIn.GetActiveWorksheet();
            Range range = worksheet.UsedRange;

            int totalDeColunas = range.Columns.Count;

            List<string> colunasComInformacoes = new List<string>();

            for (int i = 1; i <= totalDeColunas; i++)
            {
                if (range.Cells[1, i].Value2 != null)
                {
                    string enderecoDaCuluna = range.Columns[i].Address;
                    Regex reg = new Regex(@"(\$)(\w*):");
                    if (reg.IsMatch(enderecoDaCuluna))
                    {
                        //Match match = reg.Match(enderecoDaCuluna);
                        string[] coluna = enderecoDaCuluna.Split('$');
                        colunasComInformacoes.Add(coluna[1]);
                        formacoesMap.Add(new FormacaoMap(new StringBuilder().Append("##").Append(coluna[1]).Append("##").ToString(), i));
                    }
                }
            }

            return colunasComInformacoes;
        }

        private int getPercentegem(int valor, int total)
        {
            return (valor * 100) / total;
        }
    }
}
