﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;
using System.Net.NetworkInformation;
using System.IO;
namespace MimoX.util
{
    class RequisicaoHttpUtil
    {
        public Boolean sucesso { get; private set; }
        public WebException webException { get; private set; }


        public string executarRequisicaoGet(string url)
        {
            try
            {
                if(!temInternet())
                {
                    sucesso = false;
                    return "Para utilizar este serviço precisa estar ligado à internet";
                }
                HttpWebRequest requisicao = (HttpWebRequest)WebRequest.Create(@url);
                requisicao.AutomaticDecompression = DecompressionMethods.GZip;

                using (HttpWebResponse resposta = (HttpWebResponse)requisicao.GetResponse())
                using (Stream stream = resposta.GetResponseStream())
                using (StreamReader reader = new StreamReader(stream))
                {

                    sucesso = ((int)resposta.StatusCode) >= 200 && ((int)resposta.StatusCode) < 300;
                    return reader.ReadToEnd();
                }

            }
            catch (System.Net.WebException ex)
            {
                try
                {
                    using (var stream = ex.Response.GetResponseStream())
                    using (var reader = new StreamReader(stream))
                    {
                        return reader.ReadToEnd();
                    }
                }
                catch (Exception)
                {
                    return ex.Message;
                }
            }
        }

        public string executarRequisicaoPost(string uri, string json)
        {
            try
            {
                if (!temInternet())
                {
                    sucesso = false;
                    return "Para utilizar este serviço precisa estar ligado à internet";
                }

                byte[] dataBytes = Encoding.UTF8.GetBytes(json);

                HttpWebRequest requisicao = (HttpWebRequest)WebRequest.Create(uri);
                requisicao.AutomaticDecompression = DecompressionMethods.GZip | DecompressionMethods.Deflate;
                requisicao.ContentLength = dataBytes.Length;
                requisicao.ContentType = "application/json";
                requisicao.Method = "POST";

                using (Stream requestBody = requisicao.GetRequestStream())
                {
                    requestBody.Write(dataBytes, 0, dataBytes.Length);
                }

                using (HttpWebResponse resposta = (HttpWebResponse)requisicao.GetResponse())
                using (Stream stream = resposta.GetResponseStream())
                using (StreamReader reader = new StreamReader(stream))
                {
                    sucesso = ((int)resposta.StatusCode) >= 200 && ((int)resposta.StatusCode) < 300;
                    return reader.ReadToEnd();
                }
            }
            catch (System.Net.WebException ex)
            {
                try
                {
                    using (var stream = ex.Response.GetResponseStream())
                    using (var reader = new StreamReader(stream))
                    {
                        return reader.ReadToEnd();
                    }
                }
                catch (Exception)
                {
                    return ex.Message;
                }
            }
        }

        public static void abrirLinkNoNavegador(string link)
        {
            System.Diagnostics.Process.Start(link);
        }

        public Boolean fazerPingNoServidorDeSMS()
        {
            Ping ping = new Ping();
            PingReply respostaDoPing = ping.Send("52.30.114.86");

            return respostaDoPing.Status == IPStatus.Success;

        }

        public Boolean temInternet()
        {
            return System.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable();
        }
    }
}
